@@ -1,23 +1,27 @@
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
 
+#include <QItemSelection>
+#include <QJSValue>
 #include <QMainWindow>
-#include <QPrinter>
 #include <QProgressDialog>
-#include <QScriptContext>
-#include <QScriptEngine>
-#include <QScriptValue>
 #include <QTextStream>
 
-#include "branchpropeditor.h"
-#include "extrainfodialog.h"
 #include "file.h"
 #include "flag.h"
-#include "historywindow.h"
-#include "mapeditor.h"
-#include "scripting.h"
-#include "texteditor.h"
-#include "vymview.h"
+
+#include "settings.h"
+
+class QPrinter;
+class QJSEngine;
+
+class HistoryWindow;
+class MapEditor;
+class TreeItem;
+class VymText;
+class VymModel;
+class VymView;
+class VymWrapper;
 
 class Main : public QMainWindow {
     Q_OBJECT
@@ -45,7 +49,8 @@ class Main : public QMainWindow {
     int progressCounterTotal;
 
   public:
-    void statusMessage(const QString &);
+    void logInfo(const QString &comment, const QString &caller = "");
+    void statusMessage(const QString &, int timeout = 10000);
     void setProgressMaximum(int max);
     void addProgressValue(float v);
     void initProgressCounter(uint n = 1);
@@ -65,7 +70,7 @@ class Main : public QMainWindow {
     void setupAPI();
 
     /*! Helper method to clone actions later in MapEditor */
-    void cloneActionMapEditor(QAction *a, QKeySequence ks);
+    void cloneActionMapEditor(QAction *a);
 
     void setupFileActions();
     void setupEditActions();
@@ -100,21 +105,27 @@ class Main : public QMainWindow {
   public:
     MapEditor *currentMapEditor() const;
     VymModel *currentModel() const;
-    uint currentMapID() const;
+    uint currentMapId() const;
     int currentMapIndex() const;
     VymModel *getModel(uint);
-    void gotoModel(VymModel *m);
-    void gotoModelWithID(uint id);
-    bool closeModelWithID(uint id);
+    bool gotoModel(VymModel *m);
+    bool gotoModelWithId(uint id);
+    bool closeModelWithId(uint id);
     int modelCount();
     void updateTabName(VymModel *vm);
 
   private slots:
     void editorChanged();
 
+  private:
+    int backgroundZipProcesses;
+    bool closeAfterLastZipProcess;
+
   public slots:
-    File::ErrorCode fileLoad(QString, const LoadMode &, const FileType &ftype);
-    void fileLoad(const LoadMode &);
+    void backgroundZipStarted();
+    void backgroundZipFinished();
+    bool fileLoad(QString, const File::LoadMode &, const File::FileType &ftype);
+    void fileLoad(const File::LoadMode &);
   private slots:
     void fileLoad();
     void fileSaveSession();
@@ -122,17 +133,19 @@ class Main : public QMainWindow {
     void fileRestoreSession();
   private slots:
     void fileLoadRecent();
+    void fileClearRecent();
     void addRecentMap(const QString &);
-    void fileSave(VymModel *, const SaveMode &);
+    void fileSave(VymModel *, const File::SaveMode &);
     void fileSave();
   public slots:
     void fileSave(VymModel *); // autosave from MapEditor
   private slots:
     void fileSaveAs();
-    void fileSaveAs(const SaveMode &);
+    void fileSaveAs(const File::SaveMode &);
     void fileSaveAsDefault();
     void fileImportFirefoxBookmarks();
     void fileImportFreemind();
+    void fileImportIThoughts();
     void fileImportMM();
     void fileImportDir();
     void fileExportAO();
@@ -149,17 +162,29 @@ class Main : public QMainWindow {
     void fileExportOrgMode();
     void fileExportPDF();
     void fileExportSVG();
-    void fileExportTaskjuggler();
+    void fileExportTaskJuggler();
     void fileExportXML();
     void fileExportLast();
     bool fileCloseMap(int i = -1); // Optionally pass number of tab
     void filePrint();
-    bool fileExitVYM();
+
+  public:
+    bool exitAfterScript();
+    void setExitAfterScript(bool b);
+
+  private:
+    bool exitAfterScriptInt;
+    QString repeatActionInt;
+
+  public:
+    void setRepeatAction(const QString &script);
 
   public slots:
+    bool fileExitVYM();
     void editUndo();
     void editRedo();
     void gotoHistoryStep(int);
+
   private slots:
     void editCopy();
     void editPaste();
@@ -167,20 +192,21 @@ class Main : public QMainWindow {
 
   public slots:
     void updateQueries(VymModel *);
-    bool openURL(const QString &url);
-    void openTabs(QStringList);
-    void editOpenURL();
-    void editOpenURLTab();
+    bool openUrl(const QString &url = "", bool privateMode = false);
+    void openTabs(QStringList, bool privateMode = false);
 
   private slots:
-    void editOpenMultipleVisURLTabs(bool ignoreScrolled = true);
-    void editOpenMultipleURLTabs();
+    void editOpenMultipleVisUrls(bool ignoreScrolled = true, bool privateMode = false);
+    void editOpenMultipleUrls();
+    void editOpenMultipleUrlsPrivate();
     void editNote2URLs();
     void editURL();
     void editLocalURL();
     void editHeading2URL();
+    void setJiraQuery();
     void getJiraDataSubtree();
-    void setHeadingConfluencePageName();
+    void getConfluencePageDetails();
+    void getConfluencePageDetailsRecursively();
     void getConfluenceUser();
     void openVymLinks(const QStringList &, bool background = false);
     void editVymLink();
@@ -211,16 +237,16 @@ class Main : public QMainWindow {
     void editExpandOneLevel();
     void editCollapseOneLevel();
     void editCollapseUnselected();
-    void editUnscrollChildren();
+    void editUnscrollSubtree();
     void editGrowSelectionSize();
     void editShrinkSelectionSize();
     void editResetSelectionSize();
-    void editAddAttribute();
+    void editRepeatLastAction();
     void editAddMapCenter();
-    void editNewBranch();
-    void editNewBranchBefore();
-    void editNewBranchAbove();
-    void editNewBranchBelow();
+    void editAddBranch();
+    void editAddBranchBefore();
+    void editAddBranchAbove();
+    void editAddBranchBelow();
     void editImportAdd();
     void editImportReplace();
     void editSaveBranch();
@@ -229,9 +255,9 @@ class Main : public QMainWindow {
     void editDeleteSelection();
     void editLoadImage();
     void editSaveImage();
-    void popupFollowXLink();
-    void editFollowXLink(QAction *);
     void editEditXLink(QAction *);
+    void popupFollowReference();
+    void followReference(QAction *);
 
   private slots:
     bool initLinkedMapsMenu(VymModel *model, QMenu *menu);
@@ -244,6 +270,8 @@ class Main : public QMainWindow {
     bool initTargetsMenu(VymModel *model, QMenu *menu);
     void editGoToTarget();
     void editMoveToTarget();
+    void editSelectFirstSibling();
+    void editSelectLastSibling();
     void editSelectPrevious();
     void editSelectNext();
     void editSelectNothing();
@@ -267,8 +295,7 @@ class Main : public QMainWindow {
     void formatLinkStyleParabel();
     void formatLinkStylePolyLine();
     void formatLinkStylePolyParabel();
-    void formatSelectBackColor();
-    void formatSelectBackImage();
+    void formatBackground();
     void formatSelectLinkColor();
     void formatSelectSelectionColor();
     void formatSelectFont();
@@ -283,14 +310,15 @@ class Main : public QMainWindow {
     void viewRotateClockwise();
     void viewCenter();
     void viewCenterScaled();
+    void viewCenterRotated();
 
   public slots:
     void networkStartServer();
     void networkConnect();
     void downloadFinished();
-    bool settingsPDF();
-    bool settingsURL();
-    void settingsZipTool();
+    void settingsPDF();
+    void settingsURL();
+    void settingsActionLog();
     void settingsMacroPath();
     void settingsUndoLevels();
     void settingsDefaultMapPath();
@@ -314,16 +342,17 @@ class Main : public QMainWindow {
     bool settingsConfluence();
     bool settingsJIRA();
 
-    void windowToggleNoteEditor();
-    void windowToggleTreeEditor();
+    void windowShowNoteEditor();
+    void switchEditors();
+    void windowSetTreeEditorsVisibility(bool);
     void windowToggleTaskEditor();
-    void windowToggleSlideEditor();
+    void windowShowSlideEditors();
+    void windowSetSlideEditorsVisibility(bool);
     void windowToggleScriptEditor();
     void windowToggleScriptOutput();
     void windowToggleHistory();
     void windowToggleProperty();
     void windowShowHeadingEditor();
-    void windowToggleHeadingEditor();
     void updateHistory(SimpleSettings &);
     void windowToggleAntiAlias();
     bool isAliased();
@@ -333,7 +362,7 @@ class Main : public QMainWindow {
     void updateHeading(const VymText &vt);
     void updateNoteText(const VymText &vt);
     void updateNoteEditor(TreeItem *ti);
-    void updateHeadingEditor(BranchItem *bi = nullptr);
+    void updateHeadingEditor(TreeItem *ti = nullptr);
     void selectInNoteEditor(QString s, int i);
     void setFocusMapEditor();
     void changeSelection(VymModel *model, const QItemSelection &newSel,
@@ -343,7 +372,19 @@ class Main : public QMainWindow {
     void updateActions();
     ModMode getModMode();
     bool autoSelectNewBranch();
+
+    void scriptPrint(const QString &, const QString &color = "");
     QVariant runScript(const QString &);
+    QVariant runScriptWithMacros(const QString &);
+    void abortScript(const QJSValue::ErrorType &err, const QString &msg);
+    void abortScript(const QString &msg);
+    QVariant setScriptResult(const QVariant &r);
+
+  private:
+    QJSEngine *scriptEngine;
+    QVariant scriptResult;
+
+  public slots:
     QObject *getCurrentModelWrapper();
     bool gotoWindow(const int &n);
 
@@ -365,11 +406,13 @@ class Main : public QMainWindow {
     void helpDemo();
     void helpShortcuts();
     void helpMacros();
+  public:
+    QString scriptingCommands();
+  private slots:
     void helpScriptingCommands();
     void helpDebugInfo();
     void helpAbout();
     void helpAboutQT();
-
     void callMacro();
     void downloadReleaseNotesFinished();
 
@@ -389,17 +432,15 @@ class Main : public QMainWindow {
     void checkUpdates();
     void escapePressed();
     void togglePresentationMode();
+    void toggleHideTmpMode();
 
   private:
     QString shortcutScope; //! For listing shortcuts
     QTabWidget *tabWidget;
-    qint64 *browserPID;
 
     QStringList imageTypes;
 
-    QScriptEngine scriptEngine;
-
-    QString prevSelection;
+    QUuid prevSelection;
 
     HistoryWindow *historyWindow;
 
@@ -422,6 +463,7 @@ class Main : public QMainWindow {
     QList<QAction *> actionListFiles; //! File related actions, e.g. load, save,
                                       //! restore session
     QList<QAction *> actionListBranches;
+    QList<QAction *> actionListImages;
     QList<QAction *> actionListItems;
 
     int xLinkMenuWidth;
@@ -429,6 +471,7 @@ class Main : public QMainWindow {
     QMenu *recentFilesMenu;
     enum { MaxRecentFiles = 20 };
     QAction *recentFileActions[MaxRecentFiles];
+    QAction *actionRecentFilesClear;
 
     QAction *macroActions[48];
     QStringList macro;
@@ -436,13 +479,15 @@ class Main : public QMainWindow {
     QList <QColor> quickColors;
 
     QMenu *toolbarsMenu;
+    QMenu *windowsMenu;
     QToolBar *fileToolbar;
     QToolBar *clipboardToolbar;
     QToolBar *editActionsToolbar;
     QToolBar *selectionToolbar;
     QToolBar *editorsToolbar;
     QToolBar *colorsToolbar;
-    QToolBar *zoomToolbar;
+    QToolBar *viewTransformationsToolbar;
+    QToolBar *limitedViewToolbar;
     QToolBar *modModesToolbar;
     QToolBar *referencesToolbar;
     QToolBar *standardFlagsToolbar;
@@ -455,17 +500,24 @@ class Main : public QMainWindow {
     QAction *actionFileNew;
     QAction *actionFileNewCopy;
     QAction *actionFileOpen;
+    QAction *actionFileClose;
     QAction *actionFileRestoreSession;
     QAction *actionFileSave;
     QAction *actionFilePrint;
+    QAction *actionFileExitVym;
+    QAction *actionClearRecent;
     QAction *actionMapProperties;
     QAction *actionFileExportLast;
     QAction *actionFileExportConfluence;
     QAction *actionUndo;
+    QAction *actionUndoVim;
     QAction *actionRedo;
+    QAction *actionRepeatCommand;
     QAction *actionCopy;
+    QAction *actionCopyVim;
     QAction *actionCut;
     QAction *actionPaste;
+    QAction *actionPasteVim;
     QAction *actionMoveUp;
     QAction *actionMoveDown;
     QAction *actionMoveDownDiagonally;
@@ -478,16 +530,18 @@ class Main : public QMainWindow {
     QAction *actionExpandOneLevel;
     QAction *actionCollapseOneLevel;
     QAction *actionCollapseUnselected;
-    QAction *actionOpenURL;
-    QAction *actionOpenURLTab;
-    QAction *actionOpenMultipleVisURLTabs;
-    QAction *actionOpenMultipleURLTabs;
+    QAction *actionOpenUrl;
+    QAction *actionOpenMultipleVisUrls;
+    QAction *actionOpenMultipleUrls;
+    QAction *actionOpenMultipleUrlsPrivate;
     QAction *actionGetURLsFromNote;
     QAction *actionURLNew;
     QAction *actionLocalURL;
     QAction *actionHeading2URL;
     QAction *actionGetJiraDataSubtree;
-    QAction *actionGetConfluencePageName;
+    QAction *actionSetJiraQuery;
+    QAction *actionGetConfluencePageDetails;
+    QAction *actionGetConfluencePageDetailsRecursively;
     QAction *actionOpenVymLink;
     QAction *actionOpenVymLinkBackground;
     QAction *actionOpenMultipleVymLinks;
@@ -496,6 +550,7 @@ class Main : public QMainWindow {
     QAction *actionAddTimestamp;
     QAction *actionToggleTask;
     QAction *actionTogglePresentationMode;
+    QAction *actionToggleHideTmpMode;
     QAction *actionCycleTaskStatus;
     QAction *actionTaskResetDeltaPrio;
     QAction *actionTaskSleep0;
@@ -512,8 +567,7 @@ class Main : public QMainWindow {
     QAction *actionMapInfo;
     QAction *actionHeading;
     QAction *actionDelete;
-    QAction *actionDeleteAlt;
-    QAction *actionAddAttribute;
+    QAction *actionDeleteVim;
 
   public:
     QAction *actionAddMapCenter;
@@ -538,10 +592,16 @@ class Main : public QMainWindow {
     QAction *actionGoToTargetLinkedMap;
     QAction *actionGoToTarget;
     QAction *actionMoveToTarget;
+    QAction *actionSelectFirstSibling;
+    QAction *actionSelectFirstSiblingVim;
+    QAction *actionSelectLastSibling;
+    QAction *actionSelectLastSiblingVim;
     QAction *actionSelectPrevious;
     QAction *actionSelectNext;
     QAction *actionSelectNothing;
     QAction *actionFind;
+    QAction *actionFindVim;
+    QAction *actionFollowReference;
 
     QActionGroup *actionGroupQuickColors;
     QAction *actionFormatQuickColor;
@@ -549,8 +609,7 @@ class Main : public QMainWindow {
     QAction *actionFormatColorBranch;
     QAction *actionFormatColorSubtree;
     QAction *actionFormatLinkColorHint;
-    QAction *actionFormatBackColor;
-    QAction *actionFormatBackImage;
+    QAction *actionFormatBackground;
     QAction *actionFormatLinkColor;
     QAction *actionFormatSelectionColor;
     QAction *actionFormatFont;
@@ -561,7 +620,8 @@ class Main : public QMainWindow {
     QAction *actionRotateCounterClockwise;
     QAction *actionRotateClockwise;
     QAction *actionCenterOn;
-    QAction *actionFitToSelection;
+    QAction *actionCenterOnScaled;
+    QAction *actionCenterOnRotated;
 
     QActionGroup *actionGroupModModes;
     QAction *actionModModePoint;
@@ -586,9 +646,9 @@ class Main : public QMainWindow {
 
     QAction *actionViewToggleNoteEditor;
     QAction *actionViewToggleHeadingEditor;
-    QAction *actionViewToggleTreeEditor;
     QAction *actionViewToggleTaskEditor;
-    QAction *actionViewToggleSlideEditor;
+    QAction *actionViewSwitchEditors;
+    QAction *actionViewShowSlideEditors;
     QAction *actionViewToggleScriptEditor;
     QAction *actionViewToggleScriptOutput;
     QAction *actionViewToggleHistoryWindow;
@@ -599,7 +659,6 @@ class Main : public QMainWindow {
 
     QAction *actionConnectGetConfluenceUser;
     QAction *actionSettingsAutoSelectNewBranch;
-    QAction *actionSettingsAutoSelectText;
     QAction *actionSettingsUseFlagGroups;
     QAction *actionSettingsUseHideExport;
     QAction *actionSettingsToggleAutosave;
